@@ -44,7 +44,7 @@ use Carp qw(croak);
 use Scalar::Util qw(blessed);
 use overload ();
 
-our $VERSION = '0.03';
+our $VERSION = '0.04';
 
 method new($class: @rules) {
     my $self = bless {
@@ -97,7 +97,7 @@ fun _css_unescape_string($str) {
         $str =~ s/'\z// or die "Internal error: unterminated ' string";
     }
     $str =~ s{
-        \\ (
+        \\ (?:
             ( [[:xdigit:]]{1,6} ) (?: \r\n | [ \t\r\n\f] )?+
         |
             ( [^\n\r\f[:xdigit:]] )
@@ -173,6 +173,7 @@ fun _try_parse_simple_selector($src_ref, :$allow_tag_name) {
         }xgc
     ) {
         my ($attr, $op, $val_ident, $val_string) = ($1, $2, $3, $4);
+        $attr = _css_unescape $attr;
         $attr =~ tr/A-Z/a-z/;
         if (!defined $op) {
             return { type => ST_ATTR_HAS, attr => $attr };
@@ -1146,6 +1147,14 @@ an immediately preceding sibling element matching I<S1>.
 
 =back
 
+B<Limitation:> In the current implementation, the number of adjacent
+non-descendant combinators is limited by the number of bits that perl uses for
+integers. That is, you cannot use more than 32 (if your perl uses 32-bit
+integers) or 64 (if your perl uses 64-bit integers) simple selector sequences
+in a row if they are all joined using C<< > >>, C<~>, or C<+>. (No limit is
+placed on the number of simple selectors in a sequence, nor on simple selector
+sequences joined using the descendant combinator (whitespace).)
+
 A simple selector sequence is a sequence of one or more simple selectors
 separated by nothing (not even whitespace). If a universal or type selector is
 present, it must come first in the sequence. A sequence matches any element
